%
% gene.m :
%
% This is the lab genetic algorithm that has been revised by Yevgeniy
% Sirotin (keeping the original functionalily).  It is designed to fit
% your model using a simple genetic approach.  The algorithm treats your
% model like an individual with your model's output being that
% individual's fitness value.  It creates a population of models and then
% determines which are best fit.  All the model fitnesses and parameters
% are saved in a file.  The program then selects the best fit individuals
% from this population and proceeds to breed and mutate them to produce a
% new generation.  This is done for a certain number of generations.  The
% genetic algorithm then is finished. 
%
% In order to use this genetic algorithm, your model must comply with a
% few regulations:
%
% 1.  The output of the model should be its fitness value (that means you
%     have to write a fitness generating function yourself)
%
% 2.  The model should accept all the parameters to be used in fitting as
%     arguments, and not have any other parameters passed to it from the
%     command line (this is because gene.m runs your model using eval.m)
%
% The parameters for this model are:
%
% func        * a string containing the name of your function 
%
%               Make sure that your function is in a directory that is
%               present in matlab's path list (see addpath)
%
%               EXAMPLE: for SAM.m function should be @SAM
%
% popsize     * the population size (must be greater than 4)
%
% generations * the number of generations for which the algorithm will
%               run
%
% nparams     * number of parameters your model takes
%
% ranges      * a matrix of parameter ranges for your model (2 x
%               nParams matrix having the first row as the lower limit
%               and the second row as the top limit) These must be in
%               order of how your model accepts the parameters) 
%
% fileName    * the path of the output file (if none specified, then the
%               program will use the filename: specificvalues_mean.txt
%               that will be stored in the current MATLAB working
%               directory
%
%               If the file already exists then the program will read in
%               the population from the best individuals in that file (it
%               had better be the right kind of file)
% 
% numsave     * the number of individuals to keep for mutation and mating
%               from generation to generation (if none specified then the
%               default of 1/2 popsize is used)
%
%               Numsave must be greater than 1 so that mating can take
%               place and less than popsize
%
%               you can specify to keep an actual number or a fraction
%               using the following syntax:
%
%                 numsave = 10 % saves the top 10 individuals
%                 numsave = .5 % saves 1/2 of the individuals
%
%               so if numsave is less than one then the fraction is used,
%               else the number
%

function G = gene(func,popsize,generations,nparams,ranges,fileName,numsave)
  
  disp('gene:  Genetic Algorithm Started...')
  
  % PARAMETER CHECKING
  
  if(nargin < 5 | nargin > 7) % if the number of parameters entered is
                              % incorrect then signal ERROR
		  
    disp('ERROR:  Incorrect number of arguments');
    disp('USAGE:  gene(func,popsize,generations,nparams,ranges)');
    disp('or');
    disp('USAGE:  gene(func,popsize,generations,nparams,ranges,fileName)');
    return
		  
  end % nargin check
  
  if(nparams ~= size(ranges,2)) % check that nParams matches ranges
    disp('ERROR:  nParams does not match the size of ranges');
    return
  end % end nParams check
  
  if(ranges(1,:) > ranges(2,:)) % check parameters bounds
    disp('ERROR:  lower bound is greater than upper bound in ranges');
    disp('        please check that the top row is less than the bottom');
    return
  end % paramter bounds check
  
  if(nargin == 5) % if the filename is not provided use default 
    fileName = 'outfile.gene';
    disp('gene:  using default filename -> outfile.gene');
  end
  
  if(popsize < 4) % if the population is too small
    disp('ERROR:  population size must be at least 4');
    return
  end
  
  if(nargin == 7) % if numsave is specified
    
    if(numsave < 1) % if fraction requested
      
      numsave = fix(popsize*numsave); % number of individuals to save
                                      % from a generaions
    end % if not then numsave is fine
  
  else % if numsave was not specified
  
    numsave=fix(popsize/2); % number of individuals to save from a
			    % generation
  end
  
  if(numsave < 2) % if the number to save is less than minimum
    disp('ERROR:  numsave must be at least 2');
    return
  end
  
  if(numsave > popsize);
    disp('ERROR:  numsave must be less than popsize');
    return
  end
  
  disp('gene:  parameters are ok');
  
  
  
  % STARTING MAIN PROGRAM
  
  % slope and intercept for making random parameter values
  RSLOPE = repmat(diff(ranges),popsize,1);
  RINT   = repmat(ranges(1,:),popsize,1);

  % initialize parameters 
  P = zeros(popsize,nparams); % population matrix
  bestfit = Inf;              % current bestfit
  
  disp('Working...');
  
  % loop for number of generations (starting work)
  for(gen=1:1:generations)

    % sets parameter values for population
    if( exist(fileName) & (gen == 1) )
      
      % uses sp vals file to set params
      
      % update gen to correct generation 
      % (length of sp val file / popsize rounded down to integer)
      
      in = fopen(fileName,'rt'); % read in values
      M = fscanf(in,'%f',[nparams+1,inf])';
      fclose(in);
      
      if(size(M,1)<popsize) % if problem w. file
	fprintf('ERROR:  not enough individuals in file: %s\n',fileName);
	disp('please provide at least popsize individuals or remove the file');
	return
      end
      
      M = sortrows(M,1);      % sort by erfval
      P = M(1:popsize,2:end); % take top popsize values as new population
      
    elseif(gen > 1)
      
      P = newpop; % use new population from evolve to set params
      
      % else, set params from scratch (uniform dist within param range)
    elseif( ~exist(fileName) & (gen==1))
      
      
      %P is the randomly generated parameter matrix
      P = RINT + rand(popsize,nparams) .* RSLOPE;
      
    end
    
    % loop for size of population
    for(pop=1:1:popsize)
      
      % sends subject values and param values for an individual to sim function
      % sim function returns erfval
      erfval(pop,1) = feval(func,P(pop,:));
      
      if(erfval(pop,1) < bestfit(1,1))
	
	bestfit = [ erfval(pop,1) P(pop,:) ];
	    
      end
      
      % writes to specific values file
      outfile = fopen(fileName,'at');
      OP = [ erfval(pop,1) P(pop,:) ];
      fprintf(outfile,'%2.4f\t',OP);
      fprintf(outfile,'\n');

      fclose(outfile);
      
    end % popsize loop
    
    % sends population with erf values to g.a. function
    % g.a. function returns new population
    newpop = evolve(popsize,numsave,ranges,fileName);
    
  end % generations loop

  G = 'done';


% ***** START OF EVOLVE ******  

  
% evolve.m
%
% evolve(popsize,numsave,ranges)
%
% popsize- number of individuals total
% numsave- number of individuals to keep (not evolve)
% ranges- holds the ranges for all the parameters
% 
% evolves population and returns new population
% sorts individuals by erfval
% recombines top half to replace bottom half
% mutates top half
% returns new population, in M
%

function M = evolve(popsize,numsave,ranges,fileName)
  
  mutation_percent=0.5; % sigma for the mutation
  nparams=size(ranges,2); % compute the number of parameters to evolve

  % read in specific values file
  in=fopen(fileName,'rt');
  M=fscanf(in,'%f',[nparams+1,inf])';
  fclose(in);
  
  % sort individuals by erfval from last generation only
  M=M((end-popsize+1):end,:); 
  M=sortrows(M,1);

  % recombination
  for(i=1:1:(popsize-numsave))
    
    p1=zeros(1,nparams+2);
    p2=zeros(1,nparams+2);
    offspr=zeros(1,nparams+2);
    
    timeout = 0;
    % check that offspring is different from parents
    while((isequal(offspr,p1) | isequal(offspr,p2)) & timeout < 1000)
      timeout = timeout+1;
      % pick random parents 
      parents = randperm(numsave);
      parent(1:2) = parents(1:2);
    
      % randomly select a parent for each parameter
      for(p=1:1:nparams)
	gene_owner = round(rand)+1; % select either parent 1 or 2 
	M(i+numsave,p+1) = M(parent(gene_owner),p+1);
      end
      
      % set parents and offspring
      p1 = M(parent(1),2:end);
      p2 = M(parent(2),2:end);
      offspr = M(i+numsave,2:end);
    end
    
  end
  
  % mutation  
  for(i=1:1:numsave)
    
    % randomly select a param and add a random normal amount
    params = randperm(nparams);
    param = params(1);
    
    % make sure it chooses a new value
    oldparam=M(i,param+1);
    
    timeout = 0;
    while((M(i,param+1) == oldparam) & timeout < 1000)
      timeout = timeout + 1;
      
      M(i,param+1) = M(i,param+1) + ranges(1,param) + diff(ranges(:,param)) ...
	  * randn * (mutation_percent * M(i,param+1)); 
      
      if(M(i,param+1) < ranges(1,param))
	
	M(i,param+1)=ranges(1,param); % truncate
      
      elseif(M(i,param+1) > ranges(2,param)) 
	
	M(i,param+1)=ranges(2,param); 
      
      end
      
    end
    
  end % mutation loop

M=M(:,2:end);
